import ply.lex as lex
from deep_translator import (GoogleTranslator,DeeplTranslator)
import re


class Lexer:
    t_BLADE_COMMENT = r'\(?{{--.*--}}\)?'
    t_BLADE_EXPRESSION = r'@[a-z]+.+'
    t_BLADE_ECHO = r'[.;,]*(({{.*?}})|(\({{.*?}}\))|({!!.+!!}))[.;,]*'
    t_SPECIAL_CHARACTERS = r'[=\/\\\%\^\&\*\-\+\[\]]+'
    t_NUMBER = r'[0-9]+(.[0-9]*)?'
    tokens = (
            'DATE',
            'WHITESPACE',
            'EMAIL',
            'BLADE_COMMENT',
            'BLADE_ECHO',
            'STRING',
            'BLADE_EXPRESSION',
            'NUMBER',
            'SPECIAL_CHARACTERS',
        )
    def __init__(self, target_languages={'es': GoogleTranslator(source='auto', target='es'), 'en': GoogleTranslator(source='auto', target='en')}, default_language='en', **kwargs):
        self.translations = {key: {} for key in target_languages.keys()}
        self.target_languages = [None]*len(target_languages)
        i = 1
        for target_language, translator in target_languages.items():
            if target_language == default_language:
                self.target_languages[0] = (target_language, translator)
            else:
                self.target_languages[i] = (target_language, translator)
                i += 1
        self.lexer = lex.lex(object=self, **kwargs)

    def t_EMAIL(self,t):
        r'[\w.-]+@([\w-]+.)+[\w-]{2,4}'
        return t

    def t_DATE(self,t):
        r"((?i)\b(mon(?:day|ady|dya|dai)?|tues(?:day|ady|dya|dai)?|wed(?:nesday|ensday|nesdai)?|thurs(?:day|ady|dya|dai)?|fri(?:day|ady|dya|dai)?|sat(?:urday|erday|urady|urday)?|sun(?:day|ady|dya|dai)?)\b.* (?i)\b(jan(?:uary|uari|uaray|uare)?|feb(?:ruary|urary|ruari|ruary)?|mar(?:ch|hc)?|apr(?:il|li)?|may|jun(?:e|e)?|jul(?:y|y)?|aug(?:ust|sut|ust)?|sep(?:tember|tembre|tembr)?|oct(?:ober|obre|ober)?|nov(?:ember|embre|embr)?|dec(?:ember|embre|embr)?)\b .* \d{1,2})|\b(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) (\d{1,2}), (\d{4})\b"
        t.value = "TODO: \Carbon\Carbon::parse()"
        return t

    def t_STRING(self,t):
        r'(?i)[\sA-Za-z!?,¿.\'!"$\'áéíóúñ]+'
        text =  bytes(t.value.strip(), "utf-8").decode("unicode_escape")
        if len(text) <= 2 or re.search('(?i)lorem|ipsum|aliquip', text):
            return t
        try:
            translation = self.target_languages[0][1].translate(text=text)
            translation = re.sub(r'[\n\s\r\f]{2,}', ' ', translation)
            key = translation.lower().replace(" ", "_").replace(",", "_").replace("\"", "").replace("'", "_")
            key = re.sub(r'_{2,}', '_', key)
            key = re.sub(r'[.,;]_', '_', key)
            key = re.sub(r'(_$)|(^_)', lambda x: "", key)
            self.translations[self.target_languages[0][0]][key] = translation
            for index in range(1, len(self.target_languages)):
                target_language, translator = self.target_languages[index]
                self.translations[target_language][key] = translator.translate(text=text)
            t.value = " {{__(\"" + key + "\")}} "
            return t
        except:
            return t

    def t_WHITESPACE(self,t):
        r'[\n\r\f]+'
        t.value = " "
        return t

    def t_error(self,t):
        t.type = "DEFAULT"
        t.value = t.value[0]
        t.lexer.lexpos += 1
        return t

    def tokenize(self, text):
        self.lexer.input(text)
        while True:
            token = self.lexer.token()
            if not token:
                break
            yield token

    def translate(self, inner_content):
        acc = ""
        for token in self.tokenize(inner_content.strip()):
            acc += token.value
        return acc.strip()



from lark import Lark, Tree, Transformer, Visitor, Token

def compile(grammar):
    parser = Parser(grammar)
    return parser

def sub(grammar,replace,text):
   parser = Parser(grammar)
   return parser.sub(replace,text)

def match(grammar,text):
   parser = Parser(grammar)
   return parser.match(text)


def parse(grammar, text):
    parser = Parser(grammar)
    return parser.parse(text)


def print_pretty(grammar, text):
    print(parse(grammar, text).pretty())

class Parser(Visitor):
    def __init__(self,grammar):
        super().__init__()  
        self.matches = []
        self.parser = Lark(grammar, propagate_positions=True)
    def group(self, tree):
        self.matches.extend(tree.children)
    def match(self,text):
        tree = self.parser.parse(text)
        self.visit(tree)
        matches = [*self.matches]
        self.matches = []
        return matches
    def parse(self, text):
        return self.parser.parse(text)
    def sub(self, replace, text):
        tree = self.parser.parse(text)
        self.visit(tree)
        diff_x,diff_y = 0,0
        for match in sorted(self.matches, key=lambda token: token.start_pos):
            replacement = replace(match.value)
            start_pos = match.start_pos
            end_pos = match.end_pos
            x,y = diff_x+start_pos,diff_y+end_pos
            text = text[:x] + replacement + text[y:]
            diff = len(replacement)-(end_pos-start_pos)
            diff_x,diff_y = diff+diff_x,diff+diff_y
        self.matches = []
        return text




import argparse
import logging
import sys

from i18 import __version__, apply_i18
import json


__author__ = "sanchezcarlosjr"
__copyright__ = "sanchezcarlosjr"
__license__ = "MIT"

_logger = logging.getLogger(__name__)


class TranslateAction(argparse.Action):
    def __call__(self, parser, namespace, files, option_string=None):
        _logger.debug("Starting...")
        for file in files:
            try:
                new_text,translations = apply_i18(file.read())
            except Exception as e:
                raise Exception(f"In file {file.name} has ocurred an error: {str(e)}")
            if namespace.save and translations != {'es': {}, 'en': {}}:
               with open(file.name, 'w') as writer:
                   writer.write(new_text)
            print(json.dumps(translations))
        _logger.debug("Ending...")

# ---- CLI ----
# The functions defined in this section are wrappers around the main Python
# API allowing them to be called directly from the terminal as a CLI
# executable/script.


def parse_args(args):
    """Parse command line parameters

    Args:
      args (List[str]): command line parameters as list of strings
          (for example  ``["--help"]``).

    Returns:
      :obj:`argparse.Namespace`: command line parameters namespace
    """
    parser = argparse.ArgumentParser(description="Transform into i18 based on a grammar")
    parser.add_argument(
        "--version",
        action="version",
        version=f"i18 {__version__}",
    )
    parser.add_argument(
            dest="infile",
            help="file paths",
            nargs="+",
            action=TranslateAction,
            type=argparse.FileType('r'),
            default=sys.stdin
    )
    parser.add_argument(
            "-g",
            "--grammar",
            dest="grammar",
            help="file grammar",
            nargs="?",
            type=argparse.FileType('r'),
    )
    parser.add_argument(
            "-s",
            "--save",
            dest="save",
            help="Write changes in file",
            action="store_true"
    )
    parser.add_argument(
        "-v",
        "--verbose",
        dest="loglevel",
        help="set loglevel to INFO",
        action="store_const",
        const=logging.INFO,
    )
    parser.add_argument(
        "-vv",
        "--very-verbose",
        dest="loglevel",
        help="set loglevel to DEBUG",
        action="store_const",
        const=logging.DEBUG,
    )
    return parser.parse_args(args)


def setup_logging(loglevel):
    """Setup basic logging

    Args:
      loglevel (int): minimum loglevel for emitting messages
    """
    logformat = "[%(asctime)s] %(levelname)s:%(name)s:%(message)s"
    logging.basicConfig(
        level=loglevel, stream=sys.stdout, format=logformat, datefmt="%Y-%m-%d %H:%M:%S"
    )


def main(args):
    """Wrapper allowing :func:`fib` to be called with string arguments in a CLI fashion

    Instead of returning the value from :func:`fib`, it prints the result to the
    ``stdout`` in a nicely formatted message.

    Args:
      args (List[str]): command line parameters as list of strings
          (for example  ``["--verbose", "42"]``).
    """
    args = parse_args(args)
    setup_logging(args.loglevel)


def run():
    """Calls :func:`main` passing the CLI arguments extracted from :obj:`sys.argv`

    This function can be used as entry point to create console scripts with setuptools.
    """
    main(sys.argv[1:])


if __name__ == "__main__":
    run()



from string import Template

html_grammar = """
 start: element+
    
    element: /<script>((?!<\/script>).)+<\/script>/s -> script
         | /<style>((?!<\/style>).)+<\/style>/s -> style
         | "<" tag (attribute|blade_expression)* ">"
         | "<" tag (attribute|blade_expression)* "/>"
         | "@if" "(" args ")" start ("@elseif" "(" args ")" start)+ "@endif"
         | "@if" "(" args ")" start ("@elseif" "(" args ")" start)+  "@else" start "@endif"
         | switch_statement
         | "<" tag (attribute|blade_expression)* ">" element* "</" tag ">"
         | foreach
         | whitespace
         | "@if" "(" args ")" start "@endif"
         | "@if" "(" args ")" start "@else" start "@endif"
         | "@for" "(" /[^;]+;[^;]+;[^)]+/ ")" element "@endfor"
         | "@forelse"  "(" expression "as" expression ")" start "@empty" start "@endforelse"
         | "@while" "(" args? ")" start "@endwhile"
         | "@once" start "@endonce"
         | "@csrf"
         | "@push" "(" args? ")" start "@endpush"
         | "@pushOnce" "(" args? ")" start "@endPushOnce"
         | "@php" /((?!@endphp).)+/si "@endphp"
         | "@can" "(" args? ")" start "@endcan" 
         | "@auth" ( "(" args? ")" )? start "@else" start "@endauth"
         | "@guest" start "@endguest"
         | "@error" "(" args? ")" start "@enderror" 
         | "@unless" "(" args? ")" start "@endunless"
         | "@isset" "(" args? ")" start "@endisset"
         | "@component" "(" args? ")" start "@endcomponent"
         | "@break"
         | "@livewireStyles"
         | "@empty"
         | "@continue"
         | blade_expression
         | /((?!<\/?[^>]+\/?>|@elseif|@if|@for|@forelse|@endfor|@endforelse|@while|@else|@endif|@once|@crsf|@push|@pushOnce|@php|@can|@break|@continue|@case|@switch|@endswitch|@break|@default|@livewireStyles|@can|@error|@enderror|@isset|@endisset|@empty|@component|@endcomponent).)+/is -> group
    
    attribute: /[@\:]/? /[a-zA-Z][a-z:0-9A-Z.\-{}$ ]*/ /=(("{{.+?}}")|('{{.+?}}')|("[^"]*")|('[^']*'))/s? | attribute_expression
     
    attribute_expression: /[a-zA-Z@\:]+/ /\(.+\)/
    
    switch_statement: "@switch" /\([^)]+\)/ switch_cases+ "@endswitch"

    elseif_cases: "@elseif" "(" args ")" start?
    
    switch_cases: "@case" /\([^)]+\)/ start "@break" | "@default" start
    
    
    foreach: "@foreach"  "(" /((?!as).)+/ "as" /[^)]+/ ")"  start "@endforeach"
    
    blade_expression: /{{.+?}}/s | /@(?!if|for|forelse|endfor|endforelse|while|else|elseif|endif|once|crsf|push|pushOnce|php|can|break|continue|case|switch|endswitch|break|default|livewireStyles|can|error|enderror|isset|endisset|component|endcomponent)[a-z]+/si "(" args? ")" | /{!!.+?!!}/
    
    tag: /([a-z:\.\-0-9]+)/i
    
    whitespace: /[\s\r\n\t]+/x
    expression: literal
          | variable
          | unary_op expression
          | expression binary_op expression
          | method_call
          | function_call
          | "(" expression ")"
          | "[" expression "=>" expression "]"
          | variable "=>" variable
          | variable ("[" expression "]" ("[" expression "]")*)
          | expression "?" expression ":" expression

literal: ESCAPED_STRING | NUMBER |  /'.+?(?<!\\\\)'/s | /null/i | /true/i | /false/i

variable: "$" CNAME

unary_op: "!"

binary_op: "+" | "-" | "*" | "/" | "==" | "!=" | "<" | ">" | "<=" | ">=" | "%" | "===" | "!==" | "<>" | "<=>" | "&&" | "||" | "and" | "or" | "??"

method_call: (package_name "::" | variable "->" | variable "[" expression "]" | function_call "->") (CNAME "(" args? ")" | CNAME | CNAME "[" expression "]") ("->" (CNAME "(" args? ")" | CNAME | CNAME "[" expression "]"))*

 function_call: CNAME "(" args? ")"

 package_name: CNAME ("\\\\" CNAME)*

args: expression ("," expression)*

%import common.CNAME
%import common.ESCAPED_STRING
%import common.NUMBER

    COMMENT: /<!--->|<!---->|<!--.+?-->|{{--.+?--}}|<!DOCTYPE html.*?>/
    %ignore COMMENT
    %ignore /[ \r\n\t\f]+/x
"""



json_grammar = """
  start: $GROUP -> group
         | dict
         | list
         | ESCAPED_STRING
         | SIGNED_NUMBER
         | "true" | "false" | "null"

    list : "[" [start ("," start)*] "]"

    dict : "{" [pair ("," pair)*] "}"
    pair : ESCAPED_STRING ":" start

    %import common.ESCAPED_STRING
    %import common.SIGNED_NUMBER
    %import common.WS
    %ignore WS
"""

def sub_in_grammar(grammar, dictionary):
    return Template(grammar).substitute(dictionary)



import pytest

from i18.lexer import Lexer

__author__ = "sanchezcarlosjr"
__copyright__ = "sanchezcarlosjr"
__license__ = "MIT"


def get_values(tokens):
    return [(str(token), token.label_) for token in tokens]

def test_basic_expressions():
    lexer = Lexer()
    print(lexer.analyze("Hello World"))

